print("import: 'mlxtend'")
import mlxtend

print("import: 'mlxtend._base'")
import mlxtend._base

print("import: 'mlxtend.classifier'")
import mlxtend.classifier

print("import: 'mlxtend.cluster'")
import mlxtend.cluster

print("import: 'mlxtend.data'")
import mlxtend.data

print("import: 'mlxtend.evaluate'")
import mlxtend.evaluate

print("import: 'mlxtend.externals'")
import mlxtend.externals

print("import: 'mlxtend.feature_extraction'")
import mlxtend.feature_extraction

print("import: 'mlxtend.feature_selection'")
import mlxtend.feature_selection

print("import: 'mlxtend.file_io'")
import mlxtend.file_io

print("import: 'mlxtend.frequent_patterns'")
import mlxtend.frequent_patterns

print("import: 'mlxtend.math'")
import mlxtend.math

print("import: 'mlxtend.plotting'")
import mlxtend.plotting

print("import: 'mlxtend.preprocessing'")
import mlxtend.preprocessing

print("import: 'mlxtend.regressor'")
import mlxtend.regressor

print("import: 'mlxtend.text'")
import mlxtend.text

print("import: 'mlxtend.utils'")
import mlxtend.utils

